#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2022 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import yaml
import logging
import os
import json
from pathlib import Path
from fosslight_util.constant import LOGGER_NAME, SHEET_NAME_FOR_SCANNER
from typing import Tuple

_logger = logging.getLogger(LOGGER_NAME)


def write_yaml(output_file, scan_item, separate_yaml=False) -> Tuple[bool, str, str]:
    success = True
    error_msg = ""
    output = ""

    try:
        output_files = []
        output_dir = os.path.dirname(output_file)

        Path(output_dir).mkdir(parents=True, exist_ok=True)
        if separate_yaml:
            filename = os.path.splitext(os.path.basename(output_file))[0]
            separate_output_file = os.path.join(output_dir, filename)

        merge_sheet = []
        for scanner_name, _ in scan_item.file_items.items():
            sheet_name = SHEET_NAME_FOR_SCANNER[scanner_name.lower()]
            json_contents = scan_item.get_print_json(scanner_name)

            if not separate_yaml:
                merge_sheet.extend(json_contents)
            else:
                output_file = f'{separate_output_file}_{sheet_name}.yaml'
                remove_duplicates_and_dump_yaml(json_contents, output_file)
                output_files.append(output_file)

        if not separate_yaml:
            remove_duplicates_and_dump_yaml(merge_sheet, output_file)
            output_files.append(output_file)

        if output_files:
            output = ", ".join(output_files)
    except Exception as ex:
        error_msg = str(ex)
        success = False

    if not success:
        error_msg = "[Error] Writing yaml:" + error_msg

    return success, error_msg, output


def remove_duplicates_and_dump_yaml(json_contents, output_file):
    unique_json_strings = {json.dumps(e, sort_keys=True) for e in json_contents}
    unique_json_contents = [json.loads(e) for e in unique_json_strings]

    yaml_dict = {}
    for uitem in unique_json_contents:
        create_yaml_with_ossitem(uitem, yaml_dict)

    with open(output_file, 'w') as f:
        yaml.dump(yaml_dict, f, default_flow_style=False, sort_keys=False)


def create_yaml_with_ossitem(item, yaml_dict):
    item_name = item.pop("name")

    if item_name not in yaml_dict.keys():
        yaml_dict[item_name] = []
    merged = False
    for oss_info in yaml_dict[item_name]:
        if oss_info.get('version', '') == item.get('version', '') and \
           oss_info.get('license', []) == item.get('license', []) and \
           oss_info.get('copyright text', '') == item.get('copyright text', '') and \
           oss_info.get('homepage', '') == item.get('homepage', '') and \
           oss_info.get('download location', '') == item.get('download location', '') and \
           oss_info.get('exclude', False) == item.get('exclude', False):
            if isinstance(oss_info.get('source path', []), str):
                oss_info['source path'] = [oss_info.get('source path', '')]
            oss_info.get('source path', []).append(item.get('source path', ''))
            oss_info.pop('comment', None)
            merged = True
            break

    if not merged:
        yaml_dict[item_name].append(item)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2023 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import logging
import os
import json
from fosslight_util.constant import LOGGER_NAME, FOSSLIGHT_DEPENDENCY
from fosslight_util.oss_item import ScannerItem
from typing import List

logger = logging.getLogger(LOGGER_NAME)
EMPTY_FILE_PATH = '-'


def write_scancodejson(output_dir: str, output_filename: str, oss_list: List[ScannerItem]):
    json_output = {}
    json_output['headers'] = []
    json_output['summary'] = {}
    json_output['license_detections'] = []
    json_output['files'] = []
    json_output['dependencies'] = []

    for scanner, file_items in oss_list.file_items.items():
        for fi in file_items:
            if scanner == FOSSLIGHT_DEPENDENCY:
                json_output['dependencies'] = add_item_in_deps(fi, json_output['dependencies'])
            else:
                if fi.exclude:
                    continue
                if fi.oss_items and (all(oss_item.exclude for oss_item in fi.oss_items)):
                    continue
                if not fi.source_name_or_path:
                    fi.source_name_or_path = EMPTY_FILE_PATH
                json_output['files'] = add_item_in_files(fi, json_output['files'])

    with open(os.path.join(output_dir, output_filename), 'w') as f:
        json.dump(json_output, f, sort_keys=False, indent=4)


def get_oss_item_list(oss_items):
    scan_oss_items = []
    for oi in oss_items:
        if oi.exclude:
            continue
        oss_item = {}
        oss_item['name'] = oi.name
        oss_item['version'] = oi.version
        oss_item['license'] = oi.license
        oss_item['copyright'] = oi.copyright
        oss_item['download_location'] = oi.download_location
        oss_item['comment'] = oi.comment
        scan_oss_items.append(oss_item)

    return scan_oss_items


def add_item_in_files(file_item, files_list):
    files_item = {}
    files_item['path'] = file_item.source_name_or_path
    files_item['name'] = os.path.basename(file_item.source_name_or_path)
    files_item['is_binary'] = file_item.is_binary
    files_item['base_name'], files_item['extension'] = os.path.splitext(os.path.basename(file_item.source_name_or_path))
    files_item['oss'] = get_oss_item_list(file_item.oss_items)

    files_list.append(files_item)

    return files_list


def add_item_in_deps(file_item, deps_list):
    deps_item = {}
    deps_item['purl'] = file_item.purl
    deps_item['scope'] = 'dependencies'
    deps_item['depends_on'] = file_item.depends_on
    deps_item['oss'] = get_oss_item_list(file_item.oss_items)

    deps_list.append(deps_item)

    return deps_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2023 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import logging
import os
import copy
import re
from fosslight_util.constant import LOGGER_NAME, FOSSLIGHT_SOURCE
from fosslight_util.parsing_yaml import parsing_yml

logger = logging.getLogger(LOGGER_NAME)
SBOM_INFO_YAML = r"sbom(-|_)info[\s\S]*.ya?ml"


def correct_with_yaml(correct_filepath, path_to_scan, scan_item):
    success = True
    msg = ""
    correct_yaml = ""
    if correct_filepath == "":
        correct_filepath = path_to_scan

    path_to_scan = os.path.normpath(path_to_scan)
    correct_filepath = os.path.normpath(correct_filepath)
    for filename in os.listdir(correct_filepath):
        if re.search(SBOM_INFO_YAML, filename, re.IGNORECASE):
            correct_yaml = os.path.join(correct_filepath, filename)
            break
    if not correct_yaml:
        msg = f"Cannot find sbom-info.yaml in {correct_filepath}."
        success = False
        return success, msg, scan_item

    rel_path = os.path.relpath(path_to_scan, correct_filepath)

    yaml_file_list, _, err_msg = parsing_yml(correct_yaml, os.path.dirname(correct_yaml), print_log=True)
    find_match = False
    for scanner_name, _ in scan_item.file_items.items():
        correct_fileitems = []
        exclude_fileitems = []
        for yaml_file_item in yaml_file_list:
            yaml_path_exists = False
            if yaml_file_item.source_name_or_path == '':
                if scanner_name == FOSSLIGHT_SOURCE:
                    correct_item = copy.deepcopy(yaml_file_item)
                    correct_item.comment = 'Added by sbom-info.yaml'
                    correct_fileitems.append(correct_item)
                continue
            for idx, scan_file_item in enumerate(scan_item.file_items[scanner_name]):
                oss_rel_path = os.path.normpath(os.path.join(rel_path, scan_file_item.source_name_or_path))
                yi_path = yaml_file_item.source_name_or_path
                if ((os.path.normpath(yi_path) == os.path.normpath(oss_rel_path)) or
                   ((os.path.normpath(oss_rel_path).startswith(os.path.normpath(yi_path.rstrip('*')))))):
                    correct_item = copy.deepcopy(scan_file_item)
                    correct_item.exclude = yaml_file_item.exclude
                    correct_item.oss_items = copy.deepcopy(yaml_file_item.oss_items)
                    correct_item.comment = ''
                    correct_item.comment = 'Loaded from sbom-info.yaml'
                    correct_fileitems.append(correct_item)

                    yaml_path_exists = True
                    exclude_fileitems.append(idx)
            if scanner_name == FOSSLIGHT_SOURCE and not yaml_path_exists:
                correct_item = copy.deepcopy(yaml_file_item)
                if os.path.exists(os.path.normpath(yaml_file_item.source_name_or_path)):
                    correct_item.comment = 'Loaded from sbom-info.yaml'
                    correct_fileitems.append(correct_item)
                else:
                    correct_item.exclude = True
                    correct_item.comment = 'Added by sbom-info.yaml'
                    correct_fileitems.append(correct_item)
        if correct_fileitems:
            scan_item.append_file_items(correct_fileitems, scanner_name)
            find_match = True
        if exclude_fileitems:
            exclude_fileitems = list(set(exclude_fileitems))
            for e_idx in exclude_fileitems:
                scan_item.file_items[scanner_name][e_idx].exclude = True
                scan_item.file_items[scanner_name][e_idx].comment = 'Excluded by sbom-info.yaml'

    if not find_match:
        success = False
        err_msg = 'No match items in sbom-info.yaml'
        return success, err_msg, scan_item

    return success, msg, scan_item



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2024 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import sys
from fosslight_util.help import print_package_version


class CoverItem:
    tool_name_key = "Tool information"
    start_time_key = "Start time"
    python_ver_key = "Python version"
    analyzed_path_key = "Analyzed path"
    excluded_path_key = "Excluded path"
    comment_key = "Comment"

    PKG_NAMES = [
        "fosslight_scanner",
        "fosslight_source",
        "fosslight_dependency",
        "fosslight_binary"
    ]

    def __init__(self, tool_name="", start_time="", input_path="", comment="", exclude_path=[], simple_mode=True):
        if simple_mode:
            self.tool_name = f'{tool_name} v{print_package_version(tool_name, "", False)}'
        else:
            first_pkg = f'{self.PKG_NAMES[0]} v{print_package_version(self.PKG_NAMES[0], "", False)}'
            remaining_pkgs = ", ".join([
                f'{pkg_name} v{print_package_version(pkg_name, "", False)}'
                for pkg_name in self.PKG_NAMES[1:]
            ])
            self.tool_name = f'{first_pkg} ({remaining_pkgs})'

        if start_time:
            date, time = start_time.split('_')
            self.start_time = f'{date}, {time[0:2]}:{time[2:4]}'
        else:
            self.start_time = ""
        self.input_path = os.path.abspath(input_path)
        self.exclude_path = ", ".join(exclude_path)
        self.comment = comment

        self.python_version = f'{sys.version_info.major}.{sys.version_info.minor}'

    def __del__(self):
        pass

    def get_print_json(self):
        json_item = {}
        json_item[self.tool_name_key] = self.tool_name
        json_item[self.start_time_key] = self.start_time
        json_item[self.python_ver_key] = self.python_version
        json_item[self.analyzed_path_key] = self.input_path
        json_item[self.excluded_path_key] = self.exclude_path
        json_item[self.comment_key] = self.comment

        return json_item



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import logging
import os
import hashlib
from fosslight_util.constant import LOGGER_NAME, FOSSLIGHT_SCANNER
from fosslight_util.cover import CoverItem
from typing import List, Dict

_logger = logging.getLogger(LOGGER_NAME)
CHECKSUM_NULL = "0"


class OssItem:

    def __init__(self, name="", version="", license="", dl_url=""):
        self.name = name
        self.version = version
        self._license = []
        self.license = license
        self.download_location = dl_url
        self.exclude = False
        self.comment = ""
        self.homepage = ""
        self._copyright = ""

    def __del__(self):
        pass

    @property
    def license(self):
        return self._license

    @license.setter
    def license(self, value):
        if value != "":
            if not isinstance(value, list):
                value = value.split(",")
            self._license.extend(value)
        self._license = [item.strip() for item in self._license]
        self._license = list(set(self._license))

    @property
    def exclude(self):
        return self._exclude

    @exclude.setter
    def exclude(self, value):
        if value:
            self._exclude = True
        else:
            self._exclude = False

    @property
    def copyright(self):
        return self._copyright

    @copyright.setter
    def copyright(self, value):
        if value != "":
            if isinstance(value, list):
                value = "\n".join(value)
            value = value.strip()
        self._copyright = value

    @property
    def version(self):
        return self._version

    @version.setter
    def version(self, value):
        if value:
            self._version = str(value)
        else:
            self._version = ""

    @property
    def comment(self):
        return self._comment

    @comment.setter
    def comment(self, value):
        if not value:
            self._comment = ""
        else:
            if self._comment:
                self._comment = f"{self._comment} / {value}"
            else:
                self._comment = value


class FileItem:
    def __init__(self, value):
        self.relative_path = value
        self.source_name_or_path = ""
        self._exclude = False
        self._comment = ""
        self.is_binary = False
        self.oss_items: List[OssItem] = []
        self.checksum = CHECKSUM_NULL

    def __del__(self):
        pass

    @property
    def exclude(self):
        return self._exclude

    @exclude.setter
    def exclude(self, value):
        if value:
            self._exclude = True
        else:
            self._exclude = False
        for oss in self.oss_items:
            oss.exclude = value

    @property
    def comment(self):
        return self._comment

    @comment.setter
    def comment(self, value):
        if not value:
            self._comment = ""
        else:
            if self._comment:
                self._comment = f"{self._comment} / {value}"
            else:
                self._comment = value
            for oss in self.oss_items:
                oss.comment = value

    def get_print_array(self):
        items = []

        for oss in self.oss_items:
            exclude = "Exclude" if self.exclude or oss.exclude else ""
            lic = ",".join(oss.license)

            oss_item = [os.path.join(self.relative_path, self.source_name_or_path), oss.name, oss.version, lic,
                        oss.download_location, oss.homepage, oss.copyright, exclude, oss.comment]
            items.append(oss_item)
        return items

    def get_print_json(self):
        items = []

        for oss in self.oss_items:
            json_item = {}
            json_item["name"] = oss.name
            json_item["version"] = oss.version

            if self.source_name_or_path != "":
                json_item["source path"] = self.source_name_or_path
            if len(oss.license) > 0:
                json_item["license"] = oss.license
            if oss.download_location != "":
                json_item["download location"] = oss.download_location
            if oss.homepage != "":
                json_item["homepage"] = oss.homepage
            if oss.copyright != "":
                json_item["copyright text"] = oss.copyright
            if self.exclude or oss.exclude:
                json_item["exclude"] = True
            if oss.comment != "":
                json_item["comment"] = oss.comment
            items.append(json_item)
        return items


def get_checksum_sha1(source_name_or_path) -> str:
    checksum = CHECKSUM_NULL
    try:
        checksum = str(hashlib.sha1(source_name_or_path.encode()).hexdigest())
    except Exception:
        try:
            f = open(source_name_or_path, "rb")
            byte = f.read()
            checksum = str(hashlib.sha1(byte).hexdigest())
            f.close()
        except Exception as ex:
            _logger.info(f"(Error) Get_checksum: {ex}")

    return checksum


def invalid(cmd):
    _logger.info('[{}] is invalid'.format(cmd))


class ScannerItem:
    def __init__(self, pkg_name, start_time=""):
        self.cover = CoverItem(tool_name=pkg_name, start_time=start_time)
        self.file_items: Dict[str, List[FileItem]] = {pkg_name: []} if pkg_name != FOSSLIGHT_SCANNER else {}
        self.external_sheets: Dict[str, List[List[str]]] = {}

    def set_cover_pathinfo(self, input_dir, path_to_exclude):
        self.cover.input_path = input_dir
        self.cover.exclude_path = ", ".join(path_to_exclude)

    def set_cover_comment(self, value):
        if value:
            if self.cover.comment:
                self.cover.comment = f"{self.cover.comment} / {value}"
            else:
                self.cover.comment = value

    def get_cover_comment(self):
        return [item.strip() for item in self.cover.comment.split(" / ")]

    def append_file_items(self, file_item: List[FileItem], pkg_name=""):
        if pkg_name == "":
            if len(self.file_items.keys()) != 1:
                _logger.error("Package name is not set. Cannot append file_item into ScannerItem.")
            else:
                pkg_name = list(self.file_items.keys())[0]
        if pkg_name not in self.file_items:
            self.file_items[pkg_name] = []
        self.file_items[pkg_name].extend(file_item)

    def get_print_array(self, scanner_name):
        items = []
        for file_item in self.file_items[scanner_name]:
            items.extend(file_item.get_print_array())
        return items

    def get_print_json(self, scanner_name):
        items = []
        for file_item in self.file_items[scanner_name]:
            items.extend(file_item.get_print_json())
        return items

    def __del__(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2024 LG Electronics Inc.
# Copyright (c) OWASP Foundation.
# SPDX-License-Identifier: Apache-2.0

import os
import logging
import re
from pathlib import Path
from fosslight_util.constant import (LOGGER_NAME, FOSSLIGHT_DEPENDENCY, FOSSLIGHT_SCANNER,
                                     FOSSLIGHT_SOURCE)
import traceback

logger = logging.getLogger(LOGGER_NAME)

try:
    from packageurl import PackageURL
    from cyclonedx.builder.this import this_component as cdx_lib_component
    from cyclonedx.exception import MissingOptionalDependencyException
    from cyclonedx.factory.license import LicenseFactory
    from cyclonedx.model import XsUri, ExternalReferenceType
    from cyclonedx.model.bom import Bom
    from cyclonedx.model.component import Component, ComponentType, HashAlgorithm, HashType, ExternalReference
    from cyclonedx.output import make_outputter, BaseOutput
    from cyclonedx.output.json import JsonV1Dot6
    from cyclonedx.schema import OutputFormat, SchemaVersion
    from cyclonedx.validation.json import JsonStrictValidator
    from cyclonedx.output.json import Json as JsonOutputter
    from cyclonedx.validation.xml import XmlValidator
except Exception:
    logger.info('No import cyclonedx-python-lib')


def write_cyclonedx(output_file_without_ext, output_extension, scan_item):
    success = True
    error_msg = ''

    bom = Bom()
    if scan_item:
        try:
            cover_name = scan_item.cover.get_print_json()["Tool information"].split('(').pop(0).strip()
            match = re.search(r"(.+) v([0-9.]+)", cover_name)
            if match:
                scanner_name = match.group(1)
            else:
                scanner_name = FOSSLIGHT_SCANNER
        except Exception:
            cover_name = FOSSLIGHT_SCANNER
            scanner_name = FOSSLIGHT_SCANNER

        lc_factory = LicenseFactory()
        bom.metadata.tools.components.add(cdx_lib_component())
        bom.metadata.tools.components.add(Component(name=scanner_name.upper(),
                                                    type=ComponentType.APPLICATION))
        comp_id = 0
        bom.metadata.component = root_component = Component(name='Root Component',
                                                            type=ComponentType.APPLICATION,
                                                            bom_ref=str(comp_id))
        relation_tree = {}

        output_dir = os.path.dirname(output_file_without_ext)
        Path(output_dir).mkdir(parents=True, exist_ok=True)
        try:
            root_package = False
            for scanner_name, file_items in scan_item.file_items.items():
                for file_item in file_items:
                    if file_item.exclude:
                        continue
                    if scanner_name == FOSSLIGHT_SOURCE:
                        comp_type = ComponentType.FILE
                    else:
                        comp_type = ComponentType.LIBRARY

                    for oss_item in file_item.oss_items:
                        if oss_item.name == '' or oss_item.name == '-':
                            if scanner_name == FOSSLIGHT_DEPENDENCY:
                                continue
                            else:
                                comp_name = file_item.source_name_or_path
                        else:
                            comp_name = oss_item.name

                        comp_id += 1
                        comp = Component(type=comp_type,
                                         name=comp_name,
                                         bom_ref=str(comp_id))

                        if oss_item.version != '':
                            comp.version = oss_item.version
                        if oss_item.copyright != '':
                            comp.copyright = oss_item.copyright
                        if scanner_name == FOSSLIGHT_DEPENDENCY and file_item.purl:
                            comp.purl = PackageURL.from_string(file_item.purl)
                        if scanner_name != FOSSLIGHT_DEPENDENCY:
                            if file_item.checksum != '0':
                                comp.hashes = [HashType(alg=HashAlgorithm.SHA_1, content=file_item.checksum)]

                        if oss_item.download_location != '':
                            comp.external_references = [ExternalReference(url=XsUri(oss_item.download_location),
                                                                          type=ExternalReferenceType.WEBSITE)]

                        oss_licenses = []
                        for ol in oss_item.license:
                            try:
                                oss_licenses.append(lc_factory.make_from_string(ol))
                            except Exception:
                                logger.info(f'No spdx license name: {ol}')
                        if oss_licenses:
                            comp.licenses = oss_licenses

                        root_package = False
                        if scanner_name == FOSSLIGHT_DEPENDENCY:
                            if oss_item.comment:
                                oss_comment = oss_item.comment.split('/')
                                for oc in oss_comment:
                                    if oc in ['direct', 'transitive', 'root package']:
                                        if oc == 'direct':
                                            bom.register_dependency(root_component, [comp])
                                        elif oc == 'root package':
                                            root_package = True
                                            root_component.name = comp_name
                                            root_component.type = comp_type
                                            comp_id -= 1
                            else:
                                bom.register_dependency(root_component, [comp])
                            if len(file_item.depends_on) > 0:
                                purl = file_item.purl
                                relation_tree[purl] = []
                                relation_tree[purl].extend(file_item.depends_on)

                        if not root_package:
                            bom.components.add(comp)

            if len(bom.components) > 0:
                for comp_purl in relation_tree:
                    comp = bom.get_component_by_purl(PackageURL.from_string(comp_purl))
                    if comp:
                        dep_comp_list = []
                        for dep_comp_purl in relation_tree[comp_purl]:
                            dep_comp = bom.get_component_by_purl(PackageURL.from_string(dep_comp_purl))
                            if dep_comp:
                                dep_comp_list.append(dep_comp)
                        bom.register_dependency(comp, dep_comp_list)

        except Exception as e:
            success = False
            error_msg = f'Failed to create CycloneDX document object:{e}, {traceback.format_exc()}'
    else:
        success = False
        error_msg = 'No item to write in output file.'

    result_file = ''
    if success:
        result_file = output_file_without_ext + output_extension
        try:
            if output_extension == '.json':
                write_cyclonedx_json(bom, result_file)
            elif output_extension == '.xml':
                write_cyclonedx_xml(bom, result_file)
            else:
                success = False
                error_msg = f'Not supported output_extension({output_extension})'
        except Exception as e:
            success = False
            error_msg = f'Failed to write CycloneDX document: {e}'
            if os.path.exists(result_file):
                os.remove(result_file)

    return success, error_msg, result_file


def write_cyclonedx_json(bom, result_file):
    success = True
    try:
        my_json_outputter: 'JsonOutputter' = JsonV1Dot6(bom)
        my_json_outputter.output_to_file(result_file)
        serialized_json = my_json_outputter.output_as_string(indent=2)
        my_json_validator = JsonStrictValidator(SchemaVersion.V1_6)
        try:
            validation_errors = my_json_validator.validate_str(serialized_json)
            if validation_errors:
                logger.warning(f'JSON invalid, ValidationError: {repr(validation_errors)}')
        except MissingOptionalDependencyException as error:
            logger.debug(f'JSON-validation was skipped due to {error}')
    except Exception as e:
        logger.warning(f'Fail to write cyclonedx json: {e}')
        success = False
    return success


def write_cyclonedx_xml(bom, result_file):
    success = True
    try:
        my_xml_outputter: BaseOutput = make_outputter(bom=bom,
                                                      output_format=OutputFormat.XML,
                                                      schema_version=SchemaVersion.V1_6)
        my_xml_outputter.output_to_file(filename=result_file)
        serialized_xml = my_xml_outputter.output_as_string(indent=2)
        my_xml_validator = XmlValidator(SchemaVersion.V1_6)
        try:
            validation_errors = my_xml_validator.validate_str(serialized_xml)
            if validation_errors:
                logger.warning(f'XML invalid, ValidationError: {repr(validation_errors)}')
        except MissingOptionalDependencyException as error:
            logger.debug(f'XML-validation was skipped due to {error}')
    except Exception as e:
        logger.warning(f'Fail to write cyclonedx xml: {e}')
        success = False
    return success



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import xlsxwriter
import csv
import time
import logging
import os
import pandas as pd
from pathlib import Path
from fosslight_util.constant import LOGGER_NAME, SHEET_NAME_FOR_SCANNER, FOSSLIGHT_BINARY
from jsonmerge import merge

_HEADER = {'BIN (': ['ID', 'Binary Path', 'Source Code Path',
                     'NOTICE.html', 'OSS Name', 'OSS Version',
                     'License', 'Download Location', 'Homepage',
                     'Copyright Text', 'Exclude', 'Comment'],
           'SRC': ['ID', 'Source Path', 'OSS Name', 'OSS Version',
                   'License', 'Download Location', 'Homepage',
                   'Copyright Text', 'Exclude', 'Comment'],
           'BIN': ['ID', 'Binary Path', 'OSS Name', 'OSS Version',
                   'License', 'Download Location', 'Homepage',
                   'Copyright Text', 'Exclude', 'Comment',
                   'Vulnerability Link', 'TLSH', 'SHA1'],
           'DEP': ['ID', 'Package URL', 'OSS Name', 'OSS Version',
                   'License', 'Download Location', 'Homepage',
                   'Copyright Text', 'Exclude', 'Comment',
                   'Depends On']}

BIN_HIDE_HEADER = {'TLSH', "SHA1"}
_OUTPUT_FILE_PREFIX = "FOSSLight-Report_"
IDX_FILE = 0
IDX_EXCLUDE = 7
logger = logging.getLogger(LOGGER_NAME)
COVER_SHEET_NAME = 'Scanner Info'
MAX_EXCEL_URL_LENGTH = 255


def get_header_row(sheet_name, extended_header={}):
    selected_header = []

    merged_headers = merge(_HEADER, extended_header)

    selected_header = merged_headers.get(sheet_name, [])
    if not selected_header:
        for header_key in merged_headers.keys():
            if sheet_name.startswith(header_key):
                selected_header = merged_headers[header_key]
                break
    return selected_header


def write_result_to_csv(output_file, scan_item, separate_sheet=False, extended_header={}):
    success = True
    error_msg = ""
    file_extension = ".csv"
    output = ""

    try:
        output_files = []
        output_dir = os.path.dirname(output_file)
        Path(output_dir).mkdir(parents=True, exist_ok=True)
        if separate_sheet:
            filename = os.path.splitext(os.path.basename(output_file))[0]
            separate_output_file = os.path.join(output_dir, filename)

        merge_sheet = []
        for scanner_name, _ in scan_item.file_items.items():
            row_num = 1
            sheet_name = ""
            if scanner_name.lower() in SHEET_NAME_FOR_SCANNER:
                sheet_name = SHEET_NAME_FOR_SCANNER[scanner_name.lower()]
            elif extended_header:
                sheet_name = list(extended_header.keys())[0]
            sheet_content_without_header = scan_item.get_print_array(scanner_name)
            header_row = get_header_row(sheet_name, extended_header)

            if 'Copyright Text' in header_row:
                idx = header_row.index('Copyright Text')-1
                for item in sheet_content_without_header:
                    item[idx] = item[idx].replace('\n', ', ')
            if not separate_sheet:
                merge_sheet.extend(sheet_content_without_header)
                if scanner_name == list(scan_item.file_items.keys())[-1]:
                    sheet_content_without_header = merge_sheet
                else:
                    continue
            else:
                output_file = separate_output_file + "_" + sheet_name + file_extension
            try:
                sheet_content_without_header = sorted(sheet_content_without_header,
                                                      key=lambda x: (x[IDX_EXCLUDE], x[IDX_FILE] == "", x[IDX_FILE]))
            except Exception:
                pass
            with open(output_file, 'w', newline='') as file:
                writer = csv.writer(file, delimiter='\t')
                writer.writerow(header_row)
                for row_item in sheet_content_without_header:
                    row_item.insert(0, row_num)
                    writer.writerow(row_item)
                    row_num += 1
            output_files.append(output_file)
        if output_files:
            output = ", ".join(output_files)
    except Exception as ex:
        error_msg = str(ex)
        success = False

    return success, error_msg, output


def write_result_to_excel(out_file_name, scan_item, extended_header={}, hide_header={}):
    success = True
    error_msg = ""

    try:
        output_dir = os.path.dirname(out_file_name)
        Path(output_dir).mkdir(parents=True, exist_ok=True)

        workbook = xlsxwriter.Workbook(out_file_name)
        write_cover_sheet(workbook, scan_item.cover)
        if scan_item.file_items and len(scan_item.file_items.keys()) > 0:
            for scanner_name, _ in scan_item.file_items.items():
                sheet_name = ""
                if scanner_name.lower() in SHEET_NAME_FOR_SCANNER:
                    sheet_name = SHEET_NAME_FOR_SCANNER[scanner_name.lower()]
                elif extended_header:
                    sheet_name = list(extended_header.keys())[0]
                sheet_content_without_header = scan_item.get_print_array(scanner_name)
                selected_header = get_header_row(sheet_name, extended_header)
                try:
                    sheet_content_without_header = sorted(sheet_content_without_header,
                                                          key=lambda x: (x[IDX_EXCLUDE], x[IDX_FILE] == "", x[IDX_FILE]))
                except Exception:
                    pass
                if sheet_name:
                    worksheet = create_worksheet(workbook, sheet_name, selected_header)
                    write_result_to_sheet(worksheet, sheet_content_without_header)
                    if (scanner_name == FOSSLIGHT_BINARY) and (not hide_header):
                        hide_header = BIN_HIDE_HEADER
                    if hide_header:
                        hide_column(worksheet, selected_header, hide_header)

        for sheet_name, content in scan_item.external_sheets.items():
            if len(content) > 0:
                selected_header = content.pop(0)
                worksheet = create_worksheet(workbook, sheet_name, selected_header)
                write_result_to_sheet(worksheet, content)
                if hide_header:
                    hide_column(worksheet, selected_header, hide_header)

        workbook.close()
    except Exception as ex:
        error_msg = str(ex)
        success = False
    return success, error_msg


def write_cover_sheet(workbook, cover):
    worksheet = workbook.add_worksheet(COVER_SHEET_NAME)

    format_bold = workbook.add_format({'bold': True})
    worksheet.merge_range('A1:B1', 'About the scanner', format_bold)

    key_format = workbook.add_format({'bold': True, 'font_color': 'white', 'bg_color': 'navy'})
    item_format = workbook.add_format()
    item_format.set_text_wrap()

    cover_json = cover.get_print_json()
    row = 1
    for item in cover_json:
        worksheet.write(row, 0, item, key_format)
        worksheet.write(row, 1, cover_json[item], item_format)
        row += 1
    worksheet.set_column(0, 0, 30)
    worksheet.set_column(1, 1, 100)


def write_result_to_sheet(worksheet, sheet_contents):
    row = 1
    for row_item in sheet_contents:
        worksheet.write(row, 0, row)
        for col_num, value in enumerate(row_item):
            if len(value) > MAX_EXCEL_URL_LENGTH and (value.startswith("http://") or value.startswith("https://")):
                worksheet.write_string(row, col_num + 1, str(value))
            else:
                worksheet.write(row, col_num + 1, str(value))
        row += 1


def hide_column(worksheet, selected_header, hide_header):
    for col_idx, sel_hd in enumerate(selected_header):
        for hide_hd in hide_header:
            if str(sel_hd).lower() == str(hide_hd).lower():
                worksheet.set_column(col_idx, col_idx, None, None, {"hidden": True})


def create_worksheet(workbook, sheet_name, header_row):
    if len(sheet_name) > 31:
        current_time = str(time.time())
        sheet_name = current_time
    worksheet = workbook.add_worksheet(sheet_name)
    if header_row:
        for col_num, value in enumerate(header_row):
            worksheet.write(0, col_num, value)
    return worksheet


def merge_excels(find_excel_dir, final_out, merge_files='', cover=''):
    success = True
    msg = ""
    FIND_EXTENSION = '.xlsx'
    added_sheet_names = []
    try:
        files = os.listdir(find_excel_dir)

        if len([name for name in files if name.endswith(FIND_EXTENSION)]) > 0:
            writer = pd.ExcelWriter(final_out)
            write_cover_sheet(writer.book, cover)
            for file in files:
                if merge_files:
                    if file not in merge_files:
                        continue
                if file.endswith(FIND_EXTENSION):
                    f_short_name = os.path.splitext(
                        file)[0].replace(_OUTPUT_FILE_PREFIX, "")
                    file = os.path.join(find_excel_dir, file)
                    excel_file = pd.ExcelFile(file, engine='openpyxl')

                    for sheet_name in excel_file.sheet_names:
                        if sheet_name == COVER_SHEET_NAME:
                            continue
                        df_excel = pd.read_excel(
                            file, sheet_name=sheet_name, engine='openpyxl')
                        if sheet_name in added_sheet_names:
                            sheet_name = f"{f_short_name}_{sheet_name}"
                        df_excel.to_excel(writer, sheet_name, index=False)
                        added_sheet_names.append(sheet_name)

                        if sheet_name == 'BIN_FL_Binary':
                            bin_sheet = writer.sheets[sheet_name]
                            bin_sheet.set_column("L:M", None, None, {"hidden": True})  # 'TLSH', 'SHA1' column hide
            writer.close()
    except Exception as ex:
        msg = str(ex)
        success = False

    return success, msg



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2025 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import fnmatch
from typing import List


def excluding_files(patterns: List[str], path_to_scan: str) -> List[str]:
    excluded_paths = set()

    # Normalize patterns: e.g., 'sample/', 'sample/*' -> 'sample'
    # Replace backslash with slash
    normalized_patterns = []
    for pattern in patterns:
        pattern = pattern.replace('\\', '/')
        if pattern.endswith('/') or pattern.endswith('/*'):
            pattern = pattern.rstrip('/*')
        normalized_patterns.append(pattern)

    # Traverse directories
    for root, dirs, files in os.walk(path_to_scan):
        remove_dir_list = []

        # (1) Directory matching
        for d in dirs:
            dir_name = d
            dir_path = os.path.relpath(os.path.join(root, d), path_to_scan).replace('\\', '/')
            matched = False

            for pat in normalized_patterns:
                # Match directory name
                if fnmatch.fnmatch(dir_name, pat):
                    matched = True

                # Match the full relative path
                if not matched:
                    if fnmatch.fnmatch(dir_path, pat) or fnmatch.fnmatch(dir_path, pat + "/*"):
                        matched = True

                # If matched, exclude all files under this directory and stop checking patterns
                if matched:
                    sub_root_path = os.path.join(root, d)
                    for sr, _, sf in os.walk(sub_root_path):
                        for sub_file in sf:
                            sub_file_path = os.path.relpath(os.path.join(sr, sub_file), path_to_scan)
                            excluded_paths.add(sub_file_path.replace('\\', '/'))
                    remove_dir_list.append(d)
                    break

        # (1-2) Prune matched directories from further traversal
        for rd in remove_dir_list:
            dirs.remove(rd)

        # (2) File matching
        for f in files:
            file_path = os.path.relpath(os.path.join(root, f), path_to_scan).replace('\\', '/')
            for pat in normalized_patterns:
                if fnmatch.fnmatch(file_path, pat) or fnmatch.fnmatch(file_path, pat + "/*"):
                    excluded_paths.add(file_path)
                    break

    return sorted(excluded_paths)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import os
import platform
from fosslight_util.write_excel import write_result_to_excel, write_result_to_csv
from fosslight_util.write_opossum import write_opossum
from fosslight_util.write_yaml import write_yaml
from fosslight_util.write_spdx import write_spdx
from fosslight_util.write_cyclonedx import write_cyclonedx
from typing import Tuple

SUPPORT_FORMAT = {'excel': '.xlsx', 'csv': '.csv', 'opossum': '.json', 'yaml': '.yaml',
                  'spdx-yaml': '.yaml', 'spdx-json': '.json', 'spdx-xml': '.xml',
                  'spdx-tag': '.tag', 'cyclonedx-json': '.json', 'cyclonedx-xml': '.xml'}


def check_output_format(output='', format='', customized_format={}):
    success = True
    msg = ''
    output_path = ''
    output_file = ''
    output_extension = ''

    if customized_format:
        support_format = customized_format
    else:
        support_format = SUPPORT_FORMAT

    if format:
        format = format.lower()
        if format not in list(support_format.keys()):
            success = False
            msg = '(-f option) Enter the supported format: ' + ', '.join(list(support_format.keys()))
        else:
            output_extension = support_format[format]

    if success:
        if output != '':
            basename_extension = ''
            if not os.path.isdir(output):
                output_path = os.path.dirname(output)

                basename = os.path.basename(output)
                basename_file, basename_extension = os.path.splitext(basename)
            if basename_extension:
                if format:
                    if output_extension != basename_extension:
                        success = False
                        msg = f"(-o & -f option) Enter the same extension of output file(-o:'{output}') \
                                with format(-f:'{format}')."
                else:
                    if basename_extension not in support_format.values():
                        success = False
                        msg = '(-o option) Enter the supported file extension: ' + ', '.join(list(support_format.values()))
                if success:
                    output_file = basename_file
                    output_extension = basename_extension
            else:
                output_path = output

    return success, msg, output_path, output_file, output_extension


def check_output_formats(output='', formats=[], customized_format={}):
    success = True
    msg = ''
    output_path = ''
    output_files = []
    output_extensions = []

    if customized_format:
        support_format = customized_format
    else:
        support_format = SUPPORT_FORMAT

    if formats:
        # If -f option exist
        formats = [format.lower() for format in formats]
        for format in formats:
            if format not in list(support_format.keys()):
                success = False
                msg = '(-f option) Enter the supported format: ' + ', '.join(list(support_format.keys()))
            else:
                output_extensions.append(support_format[format])

    if success:
        if output != '':
            basename_extension = ''
            if not os.path.isdir(output):
                output_path = os.path.dirname(output)

                basename = os.path.basename(output)
                basename_file, basename_extension = os.path.splitext(basename)
            if basename_extension:
                if formats:
                    if basename_extension not in output_extensions:
                        success = False
                        msg = f"(-o & -f option) The format of output file(-o:'{output}') \
                                should be in the format list(-f:'{formats}')."
                else:
                    if basename_extension not in support_format.values():
                        success = False
                        msg = '(-o option) Enter the supported file extension: ' + ', '.join(list(support_format.values()))
                    output_extensions.append(basename_extension)
                output_files = [basename_file for _ in range(len(output_extensions))]
            else:
                output_path = output
    if not output_extensions:
        output_extensions = ['.xlsx']

    return success, msg, output_path, output_files, output_extensions


def check_output_formats_v2(output='', formats=[], customized_format={}):
    success = True
    msg = ''
    output_path = ''
    output_files = []
    output_extensions = []

    if customized_format:
        support_format = customized_format
    else:
        support_format = SUPPORT_FORMAT

    if formats:
        # If -f option exist
        formats = [format.lower() for format in formats]
        for format in formats:
            if format not in list(support_format.keys()):
                success = False
                msg = '(-f option) Enter the supported format with -f option: ' + ', '.join(list(support_format.keys()))
            else:
                output_extensions.append(support_format[format])

    if success:
        if output != '':
            basename_extension = ''
            if not os.path.isdir(output):
                output_path = os.path.dirname(output)

                basename = os.path.basename(output)
                basename_file, basename_extension = os.path.splitext(basename)
            if basename_extension:
                if formats:
                    if basename_extension not in output_extensions:
                        success = False
                        msg = f"(-o & -f option) The format of output file(-o:'{output}') \
                                should be in the format list(-f:'{formats}')."
                else:
                    if basename_extension not in support_format.values():
                        success = False
                        msg = '(-o option) Enter the supported file extension: ' + ', '.join(list(support_format.values()))
                    output_extensions.append(basename_extension)
                output_files = [basename_file for _ in range(len(output_extensions))]
            else:
                output_path = output
    if not output_extensions:
        output_extensions = ['.xlsx']
    if not formats:
        formats = []
        for ext in output_extensions:
            for key, value in support_format.items():
                if value == ext:
                    formats.append(key)
                    break
    return success, msg, output_path, output_files, output_extensions, formats


def write_output_file(output_file_without_ext: str, file_extension: str, scan_item, extended_header: dict = {},
                      hide_header: dict = {}, format: str = '', spdx_version: str = '2.3') -> Tuple[bool, str, str]:
    success = True
    msg = ''

    if file_extension == '':
        file_extension = '.xlsx'
    result_file = output_file_without_ext + file_extension

    if format:
        if format == 'excel':
            success, msg = write_result_to_excel(result_file, scan_item, extended_header, hide_header)
        elif format == 'csv':
            success, msg, _ = write_result_to_csv(result_file, scan_item, False, extended_header)
        elif format == 'opossum':
            success, msg = write_opossum(result_file, scan_item)
        elif format == 'yaml':
            success, msg, _ = write_yaml(result_file, scan_item, False)
        elif format.startswith('spdx') or format.startswith('cyclonedx'):
            if platform.system() == 'Windows' or platform.system() == 'Darwin':
                success = False
                msg = f'{platform.system()} not support spdx format.'
            else:
                if format.startswith('spdx'):
                    success, msg, _ = write_spdx(output_file_without_ext, file_extension, scan_item, spdx_version)
                elif format.startswith('cyclonedx'):
                    success, msg, _ = write_cyclonedx(output_file_without_ext, file_extension, scan_item)
    else:
        if file_extension == '.xlsx':
            success, msg = write_result_to_excel(result_file, scan_item, extended_header, hide_header)
        elif file_extension == '.csv':
            success, msg, result_file = write_result_to_csv(result_file, scan_item, False, extended_header)
        elif file_extension == '.json':
            success, msg = write_opossum(result_file, scan_item)
        elif file_extension == '.yaml':
            success, msg, result_file = write_yaml(result_file, scan_item, False)
        else:
            success = False
            msg = f'(-f option) Not supported file extension({file_extension})'

    return success, msg, result_file



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2022 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import uuid
import logging
import re
from pathlib import Path
from datetime import datetime
from fosslight_util.spdx_licenses import get_spdx_licenses_json, get_license_from_nick
from fosslight_util.constant import (LOGGER_NAME, FOSSLIGHT_DEPENDENCY, FOSSLIGHT_SCANNER,
                                     FOSSLIGHT_BINARY, FOSSLIGHT_SOURCE)
from fosslight_util.oss_item import CHECKSUM_NULL, get_checksum_sha1
import traceback

logger = logging.getLogger(LOGGER_NAME)

try:
    from spdx_tools.common.spdx_licensing import spdx_licensing
    from spdx_tools.spdx.model import (
        Actor,
        ActorType,
        Checksum,
        ChecksumAlgorithm,
        CreationInfo,
        Document,
        File,
        Package,
        Relationship,
        RelationshipType,
        SpdxNoAssertion,
        SpdxNone
    )
    from spdx_tools.spdx.validation.document_validator import validate_full_spdx_document
    from spdx_tools.spdx.writer.write_anything import write_file
except Exception:
    logger.info('No import spdx-tools')


def get_license_list_version():
    version = 'N/A'
    success, error_msg, licenses = get_spdx_licenses_json()
    if success:
        version = licenses['licenseListVersion']
    else:
        logger.info(f'Fail to get spdx license list version:{error_msg}')
    return version


def write_spdx(output_file_without_ext, output_extension, scan_item, spdx_version='2.3'):
    success = True
    error_msg = ''

    if scan_item:
        try:
            cover_name = scan_item.cover.get_print_json()["Tool information"].split('(').pop(0).strip()
            match = re.search(r"(.+) v([0-9.]+)", cover_name)
            if match:
                scanner_name = match.group(1)
            else:
                scanner_name = FOSSLIGHT_SCANNER
        except Exception:
            cover_name = FOSSLIGHT_SCANNER
            scanner_name = FOSSLIGHT_SCANNER
        creation_info = CreationInfo(spdx_version=f'SPDX-{spdx_version}',
                                     spdx_id='SPDXRef-DOCUMENT',
                                     name=f'SPDX Document by {scanner_name.upper()}',
                                     data_license='CC0-1.0',
                                     document_namespace=f'http://spdx.org/spdxdocs/{scanner_name.lower()}-{uuid.uuid4()}',
                                     creators=[Actor(name=cover_name, actor_type=ActorType.TOOL)],
                                     created=datetime.now())
        doc = Document(creation_info=creation_info)

        relation_tree = {}
        spdx_id_packages = []

        output_dir = os.path.dirname(output_file_without_ext)
        Path(output_dir).mkdir(parents=True, exist_ok=True)
        try:
            file_id = 0
            package_id = 0
            root_package = False
            for scanner_name, file_items in scan_item.file_items.items():
                for file_item in file_items:
                    file = ''  # file의 license, copyright은 oss item에서 append
                    if scanner_name in [FOSSLIGHT_BINARY, FOSSLIGHT_SOURCE]:
                        if file_item.exclude:
                            continue
                        if file_item.checksum == CHECKSUM_NULL:
                            if os.path.exists(file_item.source_name_or_path):
                                file_item.checksum = get_checksum_sha1(file_item.source_name_or_path)
                            if file_item.checksum == CHECKSUM_NULL:
                                logger.info(f'Failed to get checksum, Skip: {file_item.source_name_or_path}')
                                continue
                        file_id += 1
                        file = File(name=file_item.source_name_or_path,
                                    spdx_id=f'SPDXRef-File{file_id}',
                                    checksums=[Checksum(ChecksumAlgorithm.SHA1, file_item.checksum)])
                    file_license = []
                    file_copyright = []
                    file_comment = []
                    for oss_item in file_item.oss_items:
                        oss_licenses = []
                        declared_oss_licenses = []
                        lic_comment = []
                        for oi in oss_item.license:
                            oi = check_input_license_format(oi)
                            try:
                                oi_spdx = spdx_licensing.parse(oi, validate=True)
                                oss_licenses.append(oi_spdx)
                                declared_oss_licenses.append(oi)
                            except Exception:
                                logger.debug(f'No spdx license name: {oi}')
                                lic_comment.append(oi)
                                file_comment.append(oi)
                        if oss_licenses:
                            file_license.extend(oss_licenses)
                        if oss_item.copyright != '':
                            file_copyright.append(oss_item.copyright)

                        if oss_item.download_location == '':
                            if scanner_name == FOSSLIGHT_DEPENDENCY:
                                download_location = SpdxNone()
                            else:
                                continue
                        else:
                            download_location = oss_item.download_location
                        if scanner_name != FOSSLIGHT_DEPENDENCY and oss_item.name == '':
                            continue
                        package_id += 1
                        package = Package(name=oss_item.name,
                                          spdx_id=f'SPDXRef-Package{package_id}',
                                          download_location=download_location)

                        if oss_item.version != '':
                            package.version = oss_item.version

                        if scanner_name == FOSSLIGHT_DEPENDENCY:
                            package.files_analyzed = False  # If omitted, the default value of true is assumed.
                        else:
                            package.files_analyzed = True
                        if oss_item.copyright != '':
                            package.cr_text = oss_item.copyright
                        if oss_item.homepage != '':
                            package.homepage = oss_item.homepage

                        if declared_oss_licenses:
                            package.license_declared = spdx_licensing.parse(' AND '.join(declared_oss_licenses))
                        if lic_comment:
                            package.license_comment = ' '.join(lic_comment)

                        doc.packages.append(package)

                        if scanner_name == FOSSLIGHT_DEPENDENCY:
                            purl = file_item.purl
                            spdx_id_packages.append([purl, package.spdx_id])
                            relation_tree[purl] = {}
                            relation_tree[purl]['id'] = package.spdx_id
                            relation_tree[purl]['dep'] = []
                            if 'root package' in oss_item.comment:
                                root_package = True
                                relationship = Relationship(doc.creation_info.spdx_id,
                                                            RelationshipType.DESCRIBES,
                                                            package.spdx_id)
                                doc.relationships.append(relationship)
                            relation_tree[purl]['dep'].extend(file_item.depends_on)

                    if scanner_name in [FOSSLIGHT_BINARY, FOSSLIGHT_SOURCE]:
                        if file_license:
                            file.license_info_in_file = file_license
                        if file_copyright:
                            file.copyright_text = '\n'.join(file_copyright)
                        if file_comment:
                            file.license_comment = ' '.join(file_comment)
                        doc.files.append(file)

            if len(doc.packages) > 0:
                for pkg in relation_tree:
                    if len(relation_tree[pkg]['dep']) > 0:
                        pkg_spdx_id = relation_tree[pkg]['id']
                        if len(relation_tree[pkg]['dep']) > 0:
                            for pname in relation_tree[pkg]['dep']:
                                ans = next(filter(lambda x: x[0] == pname, spdx_id_packages), None)
                                if ans is None:
                                    continue
                                rel_pkg_spdx_id = ans[1]
                                relationship = Relationship(pkg_spdx_id, RelationshipType.DEPENDS_ON, rel_pkg_spdx_id)
                                doc.relationships.append(relationship)
            if not root_package:
                root_package = Package(name='root package',
                                       spdx_id='SPDXRef-ROOT-PACKAGE',
                                       download_location=SpdxNoAssertion())
                root_package.files_analyzed = False
                root_package.license_declared = SpdxNoAssertion()
                doc.packages.append(root_package)
                relationship = Relationship(doc.creation_info.spdx_id, RelationshipType.DESCRIBES, root_package.spdx_id)
                doc.relationships.append(relationship)

        except Exception as e:
            success = False
            error_msg = f'Failed to create spdx document object:{e}, {traceback.format_exc()}'
    else:
        success = False
        error_msg = 'No item to write in output file.'

    validation_messages = validate_full_spdx_document(doc)
    for message in validation_messages:
        logger.warning(message.validation_message)
        logger.warning(message.context)

    # assert validation_messages == []

    result_file = ''
    if success:
        result_file = output_file_without_ext + output_extension
        try:
            write_file(doc, result_file)
        except Exception as e:
            success = False
            error_msg = f'Failed to write spdx document: {e}'
            if os.path.exists(result_file):
                os.remove(result_file)

    return success, error_msg, result_file


def convert_to_spdx_style(input_string):
    input_string = re.sub(r'[^\w\s\.\-]', '', input_string)
    input_string = re.sub(r'[\s\_]', '-', input_string)
    input_converted = f"LicenseRef-{input_string}"
    return input_converted


def check_input_license_format(input_license):
    spdx_licenses = get_spdx_licensename()
    for spdx in spdx_licenses:
        if input_license.casefold() == spdx.casefold():
            return spdx

    if input_license.startswith('LicenseRef-'):
        return input_license

    licensesfromJson = get_license_from_nick()
    if licensesfromJson == "":
        logger.warning(" Error - Return Value to get license from Json is none")

    try:
        converted_license = licensesfromJson.get(input_license.casefold())
        if converted_license is None:
            converted_license = convert_to_spdx_style(input_license)
    except Exception as ex:
        logger.warning(f"Error - Get frequetly used license : {ex}")

    return converted_license


def get_spdx_licensename():
    spdx_licenses = []
    try:
        success, error_msg, licenses = get_spdx_licenses_json()
        if success is False:
            logger.warning(f"Error to get SPDX Licesens : {error_msg}")

        licenseInfo = licenses.get("licenses")
        for info in licenseInfo:
            shortID = info.get("licenseId")
            isDeprecated = info.get("isDeprecatedLicenseId")
            if isDeprecated is False:
                spdx_licenses.append(shortID)
    except Exception as ex:
        logger.warning(f"Error access to get_spdx_licenses_json : {ex}")
    return spdx_licenses
